/*******************************************************************************
 *	@file			TestDXLibv2.cpp
 *	@brief			DirectX Test Program
 *
 *	@date			08-Apr-2009(Wed)
 ******************************************************************************/

// vvZbT---------------------------------------------------------------
#include "DL.h"
#include "DLGraph.h"
#include "DLSound.h"
#include "DLDebug.h"

#pragma comment ( lib, "_DXLibv2.lib" )

// 񋓌^`-------------------------------------------------------------------
/*******************************************************************************
 *	@enum		enumDisplay
 *	@brief		ʃTCY
 ******************************************************************************/
enum enumDisplay
{
	enumDisplay_640x480		= 0,												//!<  VGA( 640x 480)
	enumDisplay_800x600,														//!< SVGA( 800x 600)
	enumDisplay_1024x768,														//!<  XGA(1024x 768)
	enumDisplay_FullScreen,														//!< tXN[

	enumDisplay_Max,
};

// O[oϐ---------------------------------------------------------
static const char* const APPNAME	= "TESTDXLIBV2";							//!< Av̖O
static const int APPWIDTH			= 800;										//!< Av̏
static const int APPHEIGHT			= 600;										//!< Av̏
static const float FRAMERATE		= 33.33333333333f;							//!< t[[g

DL::CDLGraph			g_CGraph;												//!< DirectX Graphics
DL::CDLSound			g_CSound;												//!< DirectX Sound
DL::CDLDebug			g_CDebug;


HANDLE	g_hMutex = NULL;														//!< Lnh
HWND	g_hWnd = NULL;															//!< EChEnh
int		g_nWidth = 0;															//!< EChE
int		g_nHeight = 0;															//!< EChE
BOOL	g_bMode = TRUE;															//!< EChE[h

enumDisplay g_eDisp = enumDisplay_800x600;										//!< \

DWORD	g_dwTimer[ 3 ];															//!< v^C}[
float	g_fDiffTime = 0.0f;														//!< ^C}[

static const int OBJMAX				= 128;
static const int DBGMAX				= 16;

int g_nBmpNum = -1;
int g_nGraNum = -1;
int	g_nWaveNum = -1;

static const int METERMAX			= 4;
static const int STRMAX				= 8;

int g_nDbgNum[ METERMAX + STRMAX ] = { -1 };


int g_nFontNum = -1;
DL::SDLDebug g_SDbg;															//!< fobO
float	g_fMeter[ 2 ];															//!< v[^[

float	g_fMinFps = FRAMERATE;													//!< vŏl
float	g_fMaxFps = 0.0f;														//!< vől
float	g_fAveFps = 0.0f;														//!< vϒl
int		g_nFrameCnt = 0;														//!< vt[

char	g_cMinFps[ 256 ];
char	g_cMaxFps[ 256 ];
char	g_cAveFps[ 256 ];

// ֐vg^Cv錾---------------------------------------------------

// -----------------------------------------------------------------------------
/*******************************************************************************
 *	@fn				LRESULT CALLBACK WndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
 *	@retval			LRESULT
 *	@brief			EChEvV[W
 ******************************************************************************/
LRESULT CALLBACK WndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
	{
	case WM_KEYDOWN:
		switch( wParam )
		{
		case VK_F2:																//!< F2L[
			if( g_hWnd != NULL )
			{
				switch( g_eDisp )
				{
				case enumDisplay_640x480:										//!< 640x480 -> 800x600
					{
						g_nWidth	= 800;
						g_nHeight	= 600;
						g_bMode		= TRUE;
						// ʐ؂ւ
						g_CGraph.ChangeDisplay( g_nWidth, g_nHeight, g_bMode );

						int nPosX = ( ::GetSystemMetrics( SM_CXSCREEN ) - g_nWidth ) / 2;
						int nPosY = ( ::GetSystemMetrics( SM_CYSCREEN ) - g_nHeight ) / 2;
						::MoveWindow( g_hWnd, nPosX, nPosY, g_nWidth, g_nHeight, TRUE );

						g_eDisp = enumDisplay_800x600;
					}
					break;
				case enumDisplay_800x600:										//!< 800x600 -> 1024x768
					{
						g_nWidth	= 1024;
						g_nHeight	= 768;
						g_bMode		= TRUE;
						// ʐ؂ւ
						g_CGraph.ChangeDisplay( g_nWidth, g_nHeight, g_bMode );

						int nPosX = ( ::GetSystemMetrics( SM_CXSCREEN ) - g_nWidth ) / 2;
						int nPosY = ( ::GetSystemMetrics( SM_CYSCREEN ) - g_nHeight ) / 2;
						::MoveWindow( g_hWnd, nPosX, nPosY, g_nWidth, g_nHeight, TRUE );

						g_eDisp = enumDisplay_1024x768;
					}
					break;
				case enumDisplay_1024x768:										//!< 1024x768 -> 800x600
					{
						g_nWidth	= ::GetSystemMetrics( SM_CXSCREEN );
						g_nHeight	= ::GetSystemMetrics( SM_CYSCREEN );
						g_bMode		= FALSE;
						// ʐ؂ւ
						g_CGraph.ChangeDisplay( g_nWidth, g_nHeight, g_bMode );

						g_eDisp = enumDisplay_FullScreen;
					}
					break;
				case enumDisplay_FullScreen:									//!< tXN[ -> 640x480
					{
						g_nWidth	= 640;
						g_nHeight	= 480;
						g_bMode		= TRUE;
						// ʐ؂ւ
						g_CGraph.ChangeDisplay( g_nWidth, g_nHeight, g_bMode );

						int nPosX = ( ::GetSystemMetrics( SM_CXSCREEN ) - g_nWidth ) / 2;
						int nPosY = ( ::GetSystemMetrics( SM_CYSCREEN ) - g_nHeight ) / 2;
						::MoveWindow( g_hWnd, nPosX, nPosY, g_nWidth, g_nHeight, TRUE );

						g_eDisp = enumDisplay_640x480;
					}
					break;
				}
			}
			break;
		case VK_ESCAPE:															//!< ESCL[
			::DestroyWindow( hWnd );
			break;
		}
		break;
	case WM_DESTROY:															//!< EChE
		::PostQuitMessage( 0 );													//!< WM_QUIT𑗂
		return ::DefWindowProc( hWnd, msg, wParam, lParam );
	default :
		return ::DefWindowProc( hWnd, msg, wParam, lParam );
	}
	return ::DefWindowProc( hWnd, msg, wParam, lParam );
}

/*******************************************************************************
 *	@fn				int Init( const HINSTANCE& hInst )
 *	@param[in]		hInst			CX^Xnh
 *	@retval			int			
 *	@brief			AvP[V̏
 ******************************************************************************/
int Init( const HINSTANCE& hInst )
{
	// dŇ
	// L̊m
	g_hMutex = ::CreateMutex( NULL, FALSE, APPNAME );
	// Ľ
	if( ::WaitForSingleObject( g_hMutex, 0 ) != WAIT_OBJECT_0 )
	{
		_ASSERT( 0 && "WaitForSingleObject()" );
		::CloseHandle( g_hMutex );
		return EXIT_FAILURE;
	}

	// EChE
	WNDCLASS			wc;														//!< EChENX
	// EChENX̓o^
	wc.style				= CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc			= static_cast<WNDPROC>( WndProc );					//!< bZ[W󂯎EChE֐
	wc.cbClsExtra			= 0;
	wc.cbWndExtra			= 0;
	wc.hInstance			= hInst;
	wc.hIcon				= NULL;												//!< ACRo^
	wc.hCursor				= ::LoadCursor( NULL, IDC_ARROW );					//!< J[\o^
	wc.hbrBackground		= static_cast<HBRUSH>( ::GetStockObject( WHITE_BRUSH ) );
	wc.lpszMenuName			= NULL;												//!< j[̓o^
	wc.lpszClassName		= APPNAME;
	// EChENX̓o^
	if( !::RegisterClass( &wc ) ) return EXIT_FAILURE;

	int nSizeX = APPWIDTH;
	int nSizeY = APPHEIGHT;
	int nPosX = ( ::GetSystemMetrics( SM_CXSCREEN ) - nSizeX ) / 2;;
	int nPosY = ( ::GetSystemMetrics( SM_CYSCREEN ) - nSizeY ) / 2;;
	// EChE쐬
	g_hWnd = ::CreateWindow( APPNAME,											//!< NX
							 APPNAME,											//!< ^Cg
							 WS_POPUP,											//!< X^C( WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU )
							 nPosX,												//!< wW
							 nPosY,												//!< xW
							 nSizeX,											//!< 
							 nSizeY,											//!< 
							 NULL,												//!< eEChẼnh
							 NULL,												//!< j[nh
							 hInst,												//!< CX^Xnh
							 NULL );											//!< lParam

	// EChE̍쐬
	if( !g_hWnd ) return EXIT_FAILURE;
	// EChE̕\Ԃ̐ݒ
	::ShowWindow( g_hWnd, SW_SHOW );
	// EChE̍XV
	::UpdateWindow( g_hWnd );

	return EXIT_SUCCESS;
}

/*******************************************************************************
 *	@fn				int Exit( void )
 *	@retval			int			
 *	@brief			AvP[V̏I
 ******************************************************************************/
int Exit( void )
{
	if( g_hWnd != NULL )
	{
		::DestroyWindow( g_hWnd );
		g_hWnd = NULL;
		::UnregisterClass( APPNAME, NULL );
	}
	if( g_hMutex != NULL ){ ::CloseHandle( g_hMutex ); g_hMutex = NULL; }
	return EXIT_SUCCESS;
}

/*******************************************************************************
 *	@fn				BOOL ProcessEvent( void )
 *	@retval			BOOL			
 *	@brief			Cxg̎s
 ******************************************************************************/
BOOL ProcessEvent( void )
{
	MSG	msg;
	// botLPOOŎgpBɂ̃AvŗDŎgp
	while( ::PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
	{
		if( !::GetMessage( &msg, NULL, 0, 0 ) )
			return FALSE;
		::TranslateMessage( &msg );
		::DispatchMessage( &msg );
	}
	return TRUE;
}

/*******************************************************************************
 *	@fn				void Init( void )
 *	@retval			BOOL			TRUE=OK/FALSE=NG
 *	@brief			
 ******************************************************************************/
BOOL Init( void )
{
	// COM̏
	::CoInitialize( NULL );

	// p[^̏
	g_hWnd		= g_hWnd;
	g_nWidth	= APPWIDTH;
	g_nHeight	= APPHEIGHT;
	g_bMode		= TRUE;

	// DLGraph̏
	if( !g_CGraph.Init( g_hWnd, g_nWidth, g_nHeight, g_bMode ) ) return FALSE;
	// DLSound̏
	if( !g_CSound.Init( g_hWnd ) ) return FALSE;

	// fobȌ
	g_SDbg.m_bUsed	= TRUE;
	RECT m_DbgRect = { APPWIDTH - 160, 0, APPWIDTH, 160 };
	g_SDbg.m_reRect = m_DbgRect;
	// DLDebug̏
	if( !g_CDebug.Init( g_SDbg ) ) return FALSE;

	{
		g_nBmpNum = g_CGraph.LoadBmp( 0, "setup.bmp", 0x00000000 );

		RECT rc = { 100, 100, 420, 340 };
		int nColor0 = 0x00000000;
		g_nGraNum = g_CGraph.AddRect( 0, 0, g_nBmpNum, DL::enumDraw_On, DL::enumState_Normal, rc, nColor0 );

		DL::D3DPLANE SPlane[ 3 ] =
		{
			{ 150, 100, 0, 0xffff0000 },
			{ 300, 300, 0, 0xff00ff00 },
			{   0, 300, 0, 0xff0000ff },
		};
		g_CGraph.AddPlane( 1, 1, -1, DL::enumDraw_On, DL::enumState_Normal, 3, SPlane );
	}

	// ǉ
	{
		g_nWaveNum = g_CSound.LoadWav( 1, "bounce.wav" );
		g_CSound.AddSE( 1, 10, g_nWaveNum, DL::enumPlay_Loop );
	}

	// fobOpIuWFNgǉ
	{
		// y
		int nColor0 = 0xCF000000;
		g_nDbgNum[ 0 ] = g_CGraph.AddRect( OBJMAX + 0, OBJMAX + 0, -1, DL::enumDraw_On, DL::enumState_Normal, m_DbgRect, nColor0 );

		// ̃C
		RECT rc1;
		rc1.left = m_DbgRect.left + 100;
		rc1.top = 0;
		rc1.right = m_DbgRect.left + 100 + 1;
		rc1.bottom = 16;
		int nColor1 = 0xCFFF0000;
		g_nDbgNum[ 1 ] = g_CGraph.AddRect( OBJMAX + 1, OBJMAX + 10, -1, DL::enumDraw_On, DL::enumState_Normal, rc1, nColor1 );

		// vO[^[
		RECT rc2;
		rc2.left = rc2.top = rc2.right = 0; rc2.bottom = 16;
		int nColor2 = 0xCF00FF00;
		g_nDbgNum[ 2 ] = g_CGraph.AddRect( OBJMAX + 2, OBJMAX + 2, -1, DL::enumDraw_On, DL::enumState_Normal, rc2, nColor2 );

		// Cu[^[
		rc2.left = rc2.top = rc2.right = 0; rc2.bottom = 16;
		int nColor3 = 0xCF0000FF;
		g_nDbgNum[ 3 ] = g_CGraph.AddRect( OBJMAX + 3, OBJMAX + 3, -1, DL::enumDraw_On, DL::enumState_Normal, rc2, nColor3 );

		// fޓo^
		g_nFontNum = g_CGraph.LoadFont( 2, "lr SVbN", 16 );

		// \p
		for( int i = 0; i < STRMAX; i++ )
		{
			int DbgNum = OBJMAX + METERMAX + i;

			rc2 = m_DbgRect;
			rc2.top = 16 + 8 + ( i + 0 ) * 16;
			rc2.bottom = 16 + 8 + ( i + 1 ) * 16;
			int nColor2 = 0xFFFFFFFF;
			LPSTR lpStr = "";

			g_nDbgNum[ METERMAX + i ] = g_CGraph.AddText( DbgNum, DbgNum, g_nFontNum, DL::enumDraw_Off, rc2, nColor2, lpStr );
		}
	}

	// ^C}[
	g_dwTimer[ 0 ] = g_dwTimer[ 1 ] = ::timeGetTime();

	return TRUE;
}

/*******************************************************************************
 *	@fn				void Proc( void )
 *	@retval			none						Ȃ
 *	@brief			vO
 ******************************************************************************/
void Proc( void )
{
	g_CDebug.Clear();
	g_CDebug.Measurement();

	// IuWFNgĐݒ
	{
		g_CSound.Play( 1 );
	}

	// \
	{
		if( g_nFrameCnt > 0 )
		{
			float fFps = g_fMeter[ 0 ] + g_fMeter[ 1 ];

			// vŏl
			int DbgNum = OBJMAX + METERMAX + 0;
			DL::D3DOBJECT* m_pD3DObj = g_CGraph.GetObject( DbgNum );
			m_pD3DObj->m_eDraw = DL::enumDraw_On;

			if( fFps < g_fMinFps ) g_fMinFps = fFps;

			DL::D3DTEXT* pText = static_cast<DL::D3DTEXT*>( m_pD3DObj->m_pObj );
			::sprintf_s( g_cMinFps, 256, "Min %.1f fps", g_fMinFps );
			pText->m_szStr = g_cMinFps;

			// vől
			DbgNum = OBJMAX + METERMAX + 1;
			m_pD3DObj = g_CGraph.GetObject( DbgNum );
			m_pD3DObj->m_eDraw = DL::enumDraw_On;

			if( g_fMaxFps < fFps ) g_fMaxFps = fFps;

			pText = static_cast<DL::D3DTEXT*>( m_pD3DObj->m_pObj );
			::sprintf_s( g_cMaxFps, 256, "Max %.1f fps", g_fMaxFps );
			pText->m_szStr = g_cMaxFps;

			// vϒl
			DbgNum = OBJMAX + METERMAX + 2;
			m_pD3DObj = g_CGraph.GetObject( DbgNum );
			m_pD3DObj->m_eDraw = DL::enumDraw_On;

			g_fAveFps = ( ( g_fAveFps * ( g_nFrameCnt - 1 ) ) + fFps ) / g_nFrameCnt;

			pText = static_cast<DL::D3DTEXT*>( m_pD3DObj->m_pObj );
			::sprintf_s( g_cAveFps, 256, "Ave %.3f fps", g_fAveFps );
			pText->m_szStr = g_cAveFps;
		}
		g_nFrameCnt++;
	}

	g_fMeter[ 0 ] = g_CDebug.Measurement();
}

/*******************************************************************************
 *	@fn				void Library( void )
 *	@retval			none						Ȃ
 *	@brief			Cu
 ******************************************************************************/
void Library( void )
{
	g_CDebug.Measurement();
	{
		// ʍXV
		g_CGraph.Update();
	}
	g_fMeter[ 1 ] = g_CDebug.Measurement();

	// [^[\
	// OvO̎s
	// PCu̎s
	// v@̊֌WA`悪P[vɂ

	float fTimer1 = 0.0f;
	float fTimer2 = 0.0f;
	float fTimer3 = 0.0f;
	// vO[^[
	{
		DL::D3DOBJECT* m_pD3DObj = g_CGraph.GetObject( OBJMAX + 2 );
		DL::D3DDATA* pData = static_cast<DL::D3DDATA*>( m_pD3DObj->m_pObj );

		DL::D3DVERTEX* v0 = static_cast<DL::D3DVERTEX*>( &pData->m_pData[ 0 ] );
		DL::D3DVERTEX* v1 = static_cast<DL::D3DVERTEX*>( &pData->m_pData[ 1 ] );
		DL::D3DVERTEX* v2 = static_cast<DL::D3DVERTEX*>( &pData->m_pData[ 2 ] );
		DL::D3DVERTEX* v3 = static_cast<DL::D3DVERTEX*>( &pData->m_pData[ 3 ] );

		fTimer1 = ( g_fMeter[ 0 ] / FRAMERATE ) * 100.0f;

		v0->x = static_cast<float>( g_SDbg.m_reRect.left + 0 );
		v1->x = static_cast<float>( g_SDbg.m_reRect.left + fTimer1 );
		v2->x = static_cast<float>( g_SDbg.m_reRect.left + fTimer1 );
		v3->x = static_cast<float>( g_SDbg.m_reRect.left + 0 );
	}
	// Cu[^[
	{
		DL::D3DOBJECT* m_pD3DObj = g_CGraph.GetObject( OBJMAX + 3 );
		DL::D3DDATA* pData = static_cast<DL::D3DDATA*>( m_pD3DObj->m_pObj );

		DL::D3DVERTEX* v0 = static_cast<DL::D3DVERTEX*>( &pData->m_pData[ 0 ] );
		DL::D3DVERTEX* v1 = static_cast<DL::D3DVERTEX*>( &pData->m_pData[ 1 ] );
		DL::D3DVERTEX* v2 = static_cast<DL::D3DVERTEX*>( &pData->m_pData[ 2 ] );
		DL::D3DVERTEX* v3 = static_cast<DL::D3DVERTEX*>( &pData->m_pData[ 3 ] );

		fTimer2 = ( g_fMeter[ 1 ] / FRAMERATE ) * 100.0f + fTimer1;

		v0->x = static_cast<float>( g_SDbg.m_reRect.left + fTimer1 );
		v1->x = static_cast<float>( g_SDbg.m_reRect.left + fTimer2 );
		v2->x = static_cast<float>( g_SDbg.m_reRect.left + fTimer2 );
		v3->x = static_cast<float>( g_SDbg.m_reRect.left + fTimer1 );
	}
}

/*******************************************************************************
 *	@fn				void Sync( void )
 *	@retval			none						Ȃ
 *	@brief			
 ******************************************************************************/
void Sync( void )
{
	// v
	while( 1 )
	{
		g_dwTimer[ 0 ] = ::timeGetTime();
		if( ( g_dwTimer[ 0 ] - g_dwTimer[ 1 ] ) >= FRAMERATE )
		{
			g_fDiffTime += static_cast<float>( ( g_dwTimer[ 0 ] - g_dwTimer[ 1 ] ) ) - FRAMERATE;
			DWORD dwDiff = static_cast<DWORD>( g_fDiffTime );
			g_fDiffTime -= dwDiff;
			g_dwTimer[ 1 ] = g_dwTimer[ 0 ] - dwDiff;
			break;
		}
		// 1ms ҂
		::Sleep( 1 );
	}
}

/*******************************************************************************
 *	@fn				void End( void )
 *	@retval			none						Ȃ
 *	@brief			I
 ******************************************************************************/
void End( void )
{
	// IuWFNg폜
	g_CGraph.DelALL();
	g_CSound.DelALL();

	// 
	g_CGraph.RemoveALL();
	g_CSound.RemoveALL();

	// DLDebug̏I
	g_CDebug.End();

	// DLSound̏I
	g_CSound.End();
	// DLGraph̏I
	g_CGraph.End();
}

/*******************************************************************************
 *	@fn				int WINAPI WinMain( HINSTANCE hInstance,
 *									    HINSTANCE hPrevInstance,
 *										LPSTR lpCmdLine,
 *										int nCmdShow )
 *	@param[in]		hInstance		CX^Xnh
 *	@param[in]		hPrevInstance	CX^Xnh
 *	@param[in]		lpCmdLine		
 *	@param[in]		nCmdShow		
 *	@retval			LRESULT			
 *	@brief			vON
 ******************************************************************************/
int WINAPI WinMain( HINSTANCE hInstance,
				    HINSTANCE hPrevInstance,
					LPSTR lpCmdLine,
					int nCmdShow )
{
	// AvP[V̏
	if( Init( hInstance ) == EXIT_FAILURE ) return EXIT_FAILURE;

	if( !Init() ) goto ERR_EXIT;

	// [v
	do
	{
		Proc();			// vOs
		Library();		// Cus
		Sync();			// s
	} while( ProcessEvent() );

ERR_EXIT:
	End();

	// AvP[V̏I
	Exit();

	return EXIT_SUCCESS;
}
